#!/usr/bin/perl -w

use strict;
use CPAN::Test;
use CPAN::Test::Reporter;
#$CPAN::Test::Reporter::CPAN_TESTERS = 'cpan-testers@perl.org';
$CPAN::Test::Reporter::CPAN_TESTERS = 'schwern@localhost';
use File::chdir;
use File::Path;

CPAN::Test->mirror('file:/home/CPAN');
CPAN::Test->init;

my $mod_name = shift;
my $module = CPAN::Test->module_by_namespace($mod_name);

my @perls = qw(
               bleadperl-optimized
              );

$ENV{PERL5LIB} = $CWD.'/t/lib';

use vars qw($Depth);
$Depth = 0;
sub _print {
    my $tabs = "\t" x $Depth;
    print $tabs, @_, "\n";
}


for my $perl (@perls) {
    rmtree [<t/lib/*>, <t/build/*>];

    `cp t/ForceINC.pm t/lib/`;

    smoke($module, $perl);

    $module->_mistakes([]);
}

END { rmtree [<t/lib/*>, <t/build/*>]; }


sub smoke {
    my($module, $perl) = @_;
    $module->reset;

    if( $module->file =~ /^perl-5\./ ) {
        _print "Skipping perl installation";
        return 1;
    }

    my $file = $module->file;

    _print "Smoking $file against $perl";

    my $report = CPAN::Test::Reporter->new;
    $report->from('schwern@pobox.com');
    $report->which_perl($perl);
    $report->package($file);

    eval {
        $module->fetch('t/build');
        $module->unpack;
        $module->which_perl($perl);
        $module->prefix('t');
        $module->makefile_pl;

        my %prereq = $module->prereqs;
        while(my($ns, $version) = each %prereq) {
            $version = '' unless $version;
            unless( `$perl -wle 'print 1 if eval "use $ns $version; 1"'` ) {
                my $prereq = CPAN::Test->module_by_namespace($ns);
                _print "needs $ns $version";
                local $Depth = $Depth + 1;
                smoke($prereq, $perl);
            }
        }

        $module->make;
        {
            local $ENV{PERL5OPT} = '-MForceINC';
            $module->test;
        }
        $module->install;
    };
    if( $@ ) {
        my $mistake = $@;

        die "Unknown error:  $mistake" unless 
          UNIVERSAL::isa($mistake, 'CPAN::Test::Mistake');

        if( $mistake->name eq 'Old Perl' ) {
            $report->grade('na');
            $report->comments('This perl is too old for this module.');
            $report->send('schwern@pobox.com');
            _print "This perl is too old.\n";
            return 1;
        }
        else {
            _print "Fatal mistake!";
            _print $mistake->name;
            _print $module->mkpl_output;
            _print $module->make_output;
            _print $module->test_output;
            _print $mistake->more_info;
            return 0;
        }
    }

    if( $module->mistakes ) {
        my $mistake_out;
        foreach my $mistake ($module->mistakes) {
            $mistake_out .= $mistake->report;
        }
        $report->comments($mistake_out);
        _print $mistake_out;
        _print "ok, but with some non-fatal mistakes";
    }
    else {
        _print "ok";
    }

    $report->grade('pass');
    $report->test_results($module->test_output);
    $report->send('schwern@pobox.com');
    return 1;
}
