#!/usr/bin/perl -Tw

use strict;
use Test::More 'no_plan';

use CPAN::Test;

my $authors = CPAN::Test->init_authors("t/01mailrc.txt");
is( keys %$authors, 1640,    'init_authors returned a bunch of authors' );

ok( !grep(!$_->isa('CPAN::Test::Author'), values %$authors),
                             "they're all CPAN::Test::Authors" );

my $schwern = CPAN::Test->author('MSCHWERN');

is( $schwern->id,    'MSCHWERN',             '  id is right' );
is( $schwern->dir,   'M/MS/MSCHWERN',        '  dir' );
is( $schwern->email, 'schwern@pobox.com',    '  email' );
is( $schwern->name,  'Michael G Schwern',   '  and name'  );

ok( eq_set([$schwern->modules], []),          "The author has no modules" );

CPAN::Test->mirror('file:t/CPAN/');
my $modules = CPAN::Test->init_packages("t/02packages.details.txt");
is( keys %$modules, 2819,  'init_packages returned a bunch of modules' );

my $sex = CPAN::Test->module_by_archive('Sex-0.69.tar.gz');

isa_ok( $sex, 'CPAN::Test::Module' );
ok( eq_hash( {$sex->namespaces}, {Sex => 0.69} ),  '  namespace' );
is( $sex->author->id,  'MSCHWERN',                 '  author' );
is( $sex->file,        'Sex-0.69.tar.gz',          '  file'   );
is( $sex->filepath,    'M/MS/MSCHWERN/Sex-0.69.tar.gz',  '  filepath' );
is( $sex->which_mirror,'file:t/CPAN/',             '  mirror' );

my $ai = CPAN::Test->module_by_namespace('AI::Fuzzy');

isa_ok( $ai, 'CPAN::Test::Module' );
ok( eq_hash( {$ai->namespaces}, { AI::Fuzzy         => 0.01,
                                  AI::Fuzzy::Label  => undef,
                                  AI::Fuzzy::Set    => undef,
                                }),  '  multiple namespaces' );
is( $ai->author->id,    'SABREN',                   '  author' );
is( $ai->file,          'AI-Fuzzy-0.01.tar.gz',     '  file'   );
is( $ai->filepath,      'S/SA/SABREN/AI-Fuzzy-0.01.tar.gz', '  filepath' );
is( $ai->which_mirror,  'file:t/CPAN/',             '  mirror' );
is( $ai->which_perl,    $^X,                        '  which_perl' );


my $robin = CPAN::Test->author('ROBIN');

my @modules = ('Algorithm-PermuteInPlace-0.02.tar.gz',
               'PadWalker-0.03.tar.gz',
               'Want-0.05.tar.gz'
              );
ok( eq_set( \@modules, [map($_->file, $robin->modules)] ),  'Author->modules' );


@modules = CPAN::Test->read_update_email('t/CPAN_daily_update.email');
my @expected = qw( Class-PseudoHash-0.02.tar.gz
                   File-chdir-0.02.tar.gz
                   File-Searcher-0.9.tar.gz
                   File-Searcher-Interactive-0.9.tar.gz
                   HTML-FormTemplate-2.02.tar.gz
                   HTTP-DAV-0.22.tar.gz
                   IO-Filter-0.01.tar.gz
                   Math-BigInt-1.42.tar.gz
                   Math-BigInt-BitVect-1.04.tar.gz
                   Math-BigInt-GMP-1.02.tar.gz
                   Math-BigInt-Pari-1.03.tar.gz 
                   MPE-IMAGE-0.05.tar.gz
                   OurNet-BBS-1.6.tar.gz
                   Scalar-List-Utils-1.03.tar.gz
                   Tie-Hash-Cannabinol-1.02.tar.gz
                   Tie-Hash-FixedKeys-1.04.tar.gz
                   Tie-Hash-Regex-0.04.tar.gz
                  );

is( @modules, @expected, 'read_update_email got the right number of modules' );
for my $idx (0..$#expected) {
    is( $modules[$idx]->file, $expected[$idx], "  got $expected[$idx]" );
}
